#! /usr/bin/perl

my $rcs_id='RCS: $Id: faxrunqd.in,v 1.71 2006/01/13 13:58:40 gert Exp $';

use ExtUtils::testlib;
use yasdi;
use strict;
use POSIX;
use Getopt::Std;
use IO::Handle;
use vars qw($DSN $DBACCOUNT $DBPASSWORD $dbh $comm $childs %valuetypes $configfile $pidfile $yasdiini $logfile);

require "./yasdid.conf";

my $debug=0; #0: none, 1: error, 2: warning, 3: info
my $daemonmode=0;

use vars qw / $opt_d $opt_V $opt_c $opt_l $opt_D /;
$opt_d = 0;					# debug level
$opt_V = 0;					# print version number
$opt_D = 0;					# daemon
$opt_c = '';				# config file
$opt_l = '';				# log file
getopts( 'Dd:c:Vl:' ) ||
	die "Valid options: -D (daemon), -d (debug level), -c (config file), -l (log file), -V (version)\n";

if ($opt_d) {
	$debug=$opt_d;
}

if ($opt_l) {
	$logfile=$opt_l;
}

opendebuglog();
debugprint(0, "Starting up...");

if ($opt_c) {
	$configfile=$opt_c;
	debugprint (3, "using config file ".$configfile);
}

if ( $opt_V )		# print version info, and exit
{
    print <<EOF;

yasdi daemon by Stefan Helbing
$rcs_id

config read from '$configfile'

EOF
    exit 0;
}

# fork as a daemon if invoked with -D
if ( $opt_D ) {
	debugprint(3, "running in daemon mode");
	print "\nstarting up yasdid in daemon mode\n";
	$daemonmode=1;
	my $pid = fork ;
	debugprint(1, "Cannot fork: ".$!) unless defined $pid ;
	die "Cannot fork: ".$! unless defined $pid ;
	if (0 == $pid) {
		# Child process: become leader of a new session, lose STDIN, log STDOUT/ERR
		POSIX::setsid();
		umask(022);
		open(STDIN, "/dev/null") ;
		open(STDOUT, ">&LOG") ;
		open(STDERR, ">&LOG") ;
	} else {
		exit 0;     # parent process
	}
}

$childs = 0;

#
# startup... write PID file, make sure no other daemon runs
#
if ( -f $pidfile && open( FP, $pidfile ) ) {
	my $p = <FP>; chomp $p; close FP;

	if ( $p ne '' && $p != $$ )	{ # does process exist?
		if ( kill( 0 => $p ) ||	$! == EPERM ) {
			die "yasdid: already running (PID=".$p.")\n";
		} else {			# no process found
		    unlink $pidfile;
		}
	}
}
open( FP, ">".$pidfile ) || die "yasdid: can't write PID to ".$pidfile.": ".$!."\n";
print FP $$."\n";
close FP;

#
# set up handlers to handle "INT" (ctrl-c), "HUP" (hangup), "TERM" (kill)...
#    (handler function does cleanup, remove lock/pid files, etc., and exits)
#
$SIG{INT}  = \&signal_handler;			# cleanup & exit

$SIG{TERM} = \&signal_handler;			# cleanup & exit

$SIG{HUP}  = \&signal_handler_HUP;		# graceful exit
my $graceful_exit_requested = 0;

$SIG{USR1} = \&signal_handler_USR1;		# roll log file
my $roll_log_file_requested = 0;

#
# read config file
#
my %serials; #contains the assignment from serialnumber (key) to devicenumber (value)
my $pollinginterval;
my $protocoldata;
my %devicemonitoredchannels; #contains the assignment from devicenumber (key) to the names of the channels that should be observed (value)
my %devicemonitoredchannelhandles; #contains the assignment from devicenumber (key) to the handles of the channels that should be observed (value)
my $currentdevice;
my $devicecount=0;
my $monchannelcount;
my $lastdaytotal=0;

if ( open( CF, $configfile ) ) {
	while( <CF> ) {
		next if /^\s*#/;		# comment lines
		chomp;
		next if /^\s*$/;		# empty lines

		if ( /^\[DEV(\d+)\]/ ) {
			$currentdevice = $1;
			$devicecount++;
			$monchannelcount=1;
		} elsif ( /^\s*serial=(\S.*)/ ) {
			if ($currentdevice eq "") {
				die "no device specified for line $.\n";
			} else {
				$serials{$1} = $currentdevice;
			}
		} elsif ( /^\s*protocol=(\S.*)/ ) {
			$protocoldata = $1;
		} elsif ( /^\s*channel=(\S.*)/ ) {
			if ($currentdevice eq "") {
				die "no device specified for line $.\n";
			} else {
				$devicemonitoredchannels{$currentdevice}{$1}=$monchannelcount++;
			}
		} elsif ( /^\s*pollinginterval=(\d+)/ ) {
			$pollinginterval = $1;
		} elsif ( /^\s*yasdiini=(\S.*)/ ) {
			$yasdiini = $1;
		} elsif ( /^\[global\]/ ) { #will be ignored
		} else {
			debugprint(1, "syntax error in $configfile, line ".$.."!");
			die "syntax error in $configfile, line ".$.."!\n";
		}
	}
	close (CF);
} else {
	debugprint(1, "Could not open config file ".$configfile.". Please specify a correct one with parameter -c!");
	die "Could not open config file ".$configfile.". Please specify a correct one with parameter -c!\n";
}

my $inipath=$yasdiini;
$inipath=~s/(.*)\/(.*)/$1/;
if (!-e $inipath."/devices") {
	if (!mkdir($inipath."/devices",0755)) {
		debugprint(1, "Could not create devices directory ".$inipath."/devices. Yasdi library needs it to store device parameter database!");
		die "Could not create devices directory ".$inipath."/devices. Yasdi library needs it to store device parameter database!\n";
	}
}

my $maxDriverIDs;
my $drivercount;
my $handlecount;
my %serialnumbers; #contains assignment from devicehandle (key) to serialnumber (value)
#my %channelhandles;
my $i;
my $channelname;
my $channelhandle;

debugprint(3, "initalizing yasdi libraries...");
yasdi::yasdiMasterInitialize($yasdiini, $maxDriverIDs);

debugprint(3, "getting yasdi driver id list...");
$drivercount=yasdi::pl_yasdiGetDriver($maxDriverIDs);
debugprint(3, "found ".$drivercount." drivers");

debugprint(3, "activating yasdi drivers...");
for($i=0;$i<$drivercount;$i++) {
	debugprint(3, "using driver ".$i." with ID ".${yasdi::DriverIDArray}[$i]);
	my $name=yasdi::pl_yasdiGetDriverName(${yasdi::DriverIDArray}[$i]);
	my $result=yasdi::yasdiSetDriverOnline(${yasdi::DriverIDArray}[0]);
	if (!$result) {
		debugprint(1, "Driver ".$name." could not be activated.");
		exit -1;
	} else {
		debugprint(3, "Driver ".$name." has been activated.");
	}
}

while (1) { #endless loop - we need this because at least the smaller Sunny Boys are OFF overnight so if you start the daemon in this time it can't find a device...
	debugprint(3, "searching ".$devicecount." device(s)...");
	my $result=yasdi::yasdiDoMasterCmdEx("detect",$devicecount,NULL,NULL);
	if ($result==0) {
		debugprint(3, $devicecount." device(s) found");
		last;
	} else {
		debugprint(2, "could not find the required count of ".$devicecount." devices - sleeping for 1 minute");
		sleep(60);
	}
}

debugprint(3, "getting device handles...");
$handlecount=yasdi::pl_yasdiGetDeviceHandles($devicecount);
debugprint(3, "found ".$handlecount." device handles");

debugprint(3, "getting device names...");
my @channellogorder;
my @outputtitlearray;
my $outputfileexists=0;
my @protdata=split /:/,$protocoldata;
if ($protdata[0] eq "mysql") {
	use DBI;
	$DSN="dbi:mysql:database=".$protdata[2].";host=".$protdata[1];
	$DBACCOUNT=$protdata[3];
	$DBPASSWORD=$protdata[4];

	# Verbindung zur DB
	if ($dbh) {
		$dbh->disconnect;
	}
	while (1) { #endless loop - we need this because at least the smaller Sunny Boys are OFF overnight so if you start the daemon in this time it can't find a device...
		$dbh = DBI->connect($DSN, $DBACCOUNT, $DBPASSWORD, { PrintError => 1 });
		if ($dbh) {
			debugprint(3, "sucessfully connected to MySQL");
			last;
		} else {
			debugprint(2, "problems occured while connecting to MySQL - will try it again in 1 minute");
			sleep(60);
		}
	}
	$dbh->{AutoCommit}=1;
	$dbh->{RaiseError}=1;
	$dbh->{mysql_auto_reconnect}=1;
}
my $protocolfileprefix;
my $protocolfilesuffix;
if ($protdata[0] eq "file") {
	my @protparts=split /\./,$protdata[1];
	$protocolfilesuffix=pop(@protparts);
	$protocolfileprefix=join("\.",@protparts);
}

for($i=0;$i<$handlecount;$i++) {
	debugprint(3, "using handle ".$i.": ".${yasdi::HandleArray}[$i]);
	my $name=yasdi::pl_yasdiGetDeviceName(${yasdi::HandleArray}[$i]);
	$serialnumbers{${yasdi::HandleArray}[$i]}=yasdi::pl_yasdiGetDeviceSN(${yasdi::HandleArray}[$i]);
	debugprint(3, "Found device with name ".$name.", Serial Number ".$serialnumbers{${yasdi::HandleArray}[$i]});
	if (defined($serials{$serialnumbers{${yasdi::HandleArray}[$i]}})) {
		my $currentdevice=$serials{$serialnumbers{${yasdi::HandleArray}[$i]}};
		if (($protdata[0] eq "file") && (-f $protocolfileprefix."_".$currentdevice.".".$protocolfilesuffix)) {
			$outputfileexists=1;
		}
		$outputtitlearray[0]="timestamp";
		debugprint(3, "Device for Serial Number ".$serialnumbers{${yasdi::HandleArray}[$i]}." is: DEV".$currentdevice);
		debugprint(3, "getting channels for device SN".$serialnumbers{${yasdi::HandleArray}[$i]});
		yasdi::pl_yasdiGetChannelHandles(${yasdi::HandleArray}[$i],255,0x090f,0);
		my %channelhandles;
		$channelhandles{$currentdevice}=${yasdi::ChannelHandles};
		#the real handles for the channels to be monitored are taken from the device
		#and replace the placeholders ("1") assigned above
		debugprint(3, "DEV".$currentdevice.": getting handles for channels to be monitored...");
		for (keys %{$devicemonitoredchannels{$currentdevice}}) { #run over all channelnames of a device
			$channelname=$_;
			my $handle=yasdi::FindChannelName(${yasdi::HandleArray}[$i], $channelname);
			if ($handle==0) {
				debugprint(2, "DEV".$currentdevice.": didn't find the configured channel ".$channelname." for device ".$name."\nchannel will be ignored");
				undef $devicemonitoredchannels{$currentdevice}{$channelname};
			} else {
				$channellogorder[$handle]=$devicemonitoredchannels{$currentdevice}{$channelname};
				$devicemonitoredchannels{$currentdevice}{$channelname}=$handle;
			}
			$outputtitlearray[$channellogorder[$handle]]=$channelname;
		}
		if ($protdata[0] eq "mysql") {
			$comm = $dbh->prepare("SHOW TABLES LIKE '".$protdata[5]."_".$currentdevice."'");
			$comm->execute;
			if ($comm->rows==0) {
				my $createstring="CREATE TABLE `".$protdata[5]."_".$currentdevice."` (";
				for my $s (@outputtitlearray) {
					$createstring.="`".$s."` ".$valuetypes{$s}." NOT NULL,";
				}
				$createstring.="KEY `timestamp` (`timestamp`))";
				$comm = $dbh->prepare($createstring);
				$comm->execute;
			}
			#add missing columns
			for (keys %{$devicemonitoredchannels{$currentdevice}}) { #run over all channelnames of a device
				$comm = $dbh->prepare("SHOW COLUMNS FROM `".$protdata[5]."_".$currentdevice."` LIKE '".$_."'");
				$comm->execute;
				if ($comm->rows==0) {
					$comm = $dbh->prepare("ALTER TABLE `".$protdata[5]."_".$currentdevice."` ADD `".$_."` ".$valuetypes{$_}." NOT NULL");
					$comm->execute;
				}
			}
			$comm = $dbh->prepare("SHOW TABLES LIKE '".$protdata[5]."_total'");
			$comm->execute;
			if ($comm->rows==0) {
				my $createstring="CREATE TABLE `".$protdata[5]."_total` (";
				$createstring.="`datum` DATE NOT NULL,";
				$createstring.="`device` TINYINT UNSIGNED NOT NULL,";
				$createstring.="`E-Total` FLOAT NOT NULL";
				$createstring.=");";
				$comm = $dbh->prepare($createstring);
				$comm->execute;
			}
			#restart of daemon: check if we already calculated the summary for yesterday
			#yesterday's mysql date string
			my $yesterday=strftime("%Y-%m-%d", 0, 0, 0, strftime("%d",localtime())-1, strftime("%m",localtime())-1, strftime("%Y",localtime())-1900);
			#do we already have a data record for yesterday?
			$comm = $dbh->prepare("SELECT datum FROM `".$protdata[5]."_total` WHERE datum='".$yesterday."' AND device=".$currentdevice);
			$comm->execute;
			if ($comm->rows==0) { #if there doesn't exist a record for yesterday we didn't make the summary
				#today's timestamp (midnight 00:00:00)
				my $today=mktime(0, 0, 0, strftime("%d",localtime()), strftime("%m",localtime())-1, strftime("%Y",localtime())-1900);
				#get first data record from yesterday
				$comm = $dbh->prepare("SELECT `E-Total` FROM `".$protdata[5]."_".$currentdevice."` WHERE timestamp>=UNIX_TIMESTAMP('".$yesterday."') AND timestamp<'".$today."' ORDER BY timestamp ASC LIMIT 1");
				$comm->execute;
				my $firstrecord=$comm->fetchrow_array();
				#get last data record from yesterday
				$comm = $dbh->prepare("SELECT `E-Total` FROM `".$protdata[5]."_".$currentdevice."` WHERE timestamp>=UNIX_TIMESTAMP('".$yesterday."') AND timestamp<'".$today."' ORDER BY timestamp DESC LIMIT 1");
				$comm->execute;
				my $lastrecord=$comm->fetchrow_array();
				#write the result to database
				$comm = $dbh->prepare("INSERT INTO `".$protdata[5]."_total` SET `E-Total`=".($lastrecord-$firstrecord).", datum='".$yesterday."', device=".$currentdevice);
				$comm->execute;
			}
		}
		if ($protdata[0] eq "file") {
			if ($outputfileexists==0) {
				createprotocolfile($currentdevice);
			} else {
				#protocolfile already exists
				#is it an old one? Let's check the file age.
				my $filedate=(stat($protocolfileprefix."_".$currentdevice.".".$protocolfilesuffix))[9];
				my $midnight=mktime(0, 0, 0, strftime("%d",localtime()), strftime("%m",localtime())-1, strftime("%Y",localtime())-1900);
				if ($filedate<$midnight) { #last modification of file wasn't today
					my $oldfileday=strftime("%Y%m%d", localtime($filedate));
					rotateprotocolfile($currentdevice, $oldfileday);
				}
			}
		}
		#tomorrow's timestamp (midnight 00:00:00)
		#let's remember this value for the main scan loop where we check it again every day about midnight
		$lastdaytotal=mktime(0, 0, 0, strftime("%d",localtime())+1, strftime("%m",localtime())-1, strftime("%Y",localtime())-1900);
	} else {
		debugprint(2, "didn't find any device with serial number SN".$serialnumbers{${yasdi::HandleArray}[$i]}." in configuration - device will be ignored");
	}
}

# Main loop
debugprint(3, "starting main loop...");
while(1) {
    # if signalled from the user (signal USR1), we've let logrotate roll the log file and now we start a new one
    if ( $roll_log_file_requested ) {
		closedebuglog();
		$roll_log_file_requested=0;
		# start new
		opendebuglog();
    }

	# leave loop if user signalled for 'graceful exit'
	if ( $graceful_exit_requested ) {
		&signal_handler('HUP');
		last;
	}

	#regular scan for all devices
	for($i=0;$i<$handlecount;$i++) { #for all devices
		my $handle=${yasdi::HandleArray}[$i];
		do_scan($handle);
	}

	my $sleep_time=$pollinginterval;
	sleep ( $sleep_time );
}

&signal_handler('HUP');
#end

####################################################################################
#
# subroutines
#
#############

sub do_scan {
	my $handle=shift;
	my $currentdevice=$serials{$serialnumbers{$handle}};
	debugprint(3, "DEV".$currentdevice.": scanning device for new data");
	my @outputarray;
	my $dooutput=1; #standardmaessig Ausgabe vornehmen (ausser bei Fehler)
	$outputarray[0]=time();
	for (keys %{$devicemonitoredchannels{$currentdevice}}) { #run over all channelnames of a device
		$channelname=$_;
		$channelhandle=$devicemonitoredchannels{$currentdevice}{$channelname};
		my $value;
		my $valuetext;
		my $result=yasdi::pl_yasdiGetChannelValue($channelhandle,$handle,$value,$valuetext);
		if ($result==0) {
			$value=~s/^(\d+)\.(\d\d\d)(.*)/$1\.$2/; #auf die ersten drei Nachkommastellen begrenzen
			debugprint(3, "DEV".$currentdevice.": Channel ".$channelname.": Value:".$value." ".$valuetext);
			$outputarray[$channellogorder[$channelhandle]]=$value;
		} else {
			if ($result==-1) {
				debugprint(2, "DEV".$currentdevice.": invalid channelhandle for channel name ".$channelname);
			} elsif ($result==-2) {
				debugprint(2, "DEV".$currentdevice.": yasdi driver is shutting down");
			} elsif ($result==-3) {
				debugprint(2, "DEV".$currentdevice.": timeout getting a new value for channel name ".$channelname);
			} elsif ($result==-4) {
				debugprint(2, "DEV".$currentdevice.": unknown error for channel name ".$channelname.". Invalid channel value.");
			} else {
				debugprint(2, "DEV".$currentdevice.": other error(s) for channel name ".$channelname." (no answer, e.g.).");
			}
			$dooutput=0; #erfolglos: keine Ausgabe
		}
	}
	if ($dooutput==1) {
		#today's timestamp
		my $today=time();
		if ($protdata[0] eq "file") {
			if ($today>$lastdaytotal) {
				#lastdaytotal has the timestamp of the next day midnight
				#so if today is greater than lastdaytotal we got a new day and we have to rotate the protocolfile
				#yesterday's date string
				my $yesterday=strftime("%Y%m%d", 0, 0, 0, strftime("%d",localtime())-1, strftime("%m",localtime())-1, strftime("%Y",localtime())-1900);
				rotateprotocolfile($currentdevice, $yesterday);
			}
			open(OUT,">>".$protocolfileprefix."_".$currentdevice.".".$protocolfilesuffix) || print ("could not open protocol file ".$protocolfileprefix."_".$currentdevice.".".$protocolfilesuffix." for append\n");
			print OUT join(":",@outputarray)."\n";
			close(OUT);
		}
		if ($protdata[0] eq "mysql") {
			my $nullvalues=1;
			my $insertstring="INSERT INTO `".$protdata[5]."_".$currentdevice."` SET ";
			for my $s (@outputtitlearray) {
				my $v=shift(@outputarray);
				$insertstring.="`".$s."`='".$v."',";
				if (($v ne 0) && ($s ne "timestamp")) { #manchmal am Anfang des Tages werden Nullwerte von der Anlage geliefert - diese ignorieren
					$nullvalues=0;
				}
			}
			$insertstring=~s/,$//; #abschliessendes Komma entfernen
			if ($nullvalues eq 0) {
				$comm = $dbh->prepare($insertstring);
				$comm->execute;
			}
			if (($today%600)<=$pollinginterval) { #keep the mysql connection running also overnight...
				#so every 10 minutes we get a value from mysql...
				$comm = $dbh->prepare("SELECT 1");
				$comm->execute;
			}
			if ($today>$lastdaytotal) {
				#lastdaytotal has the timestamp of the next day midnight
				#so if today is greater than lastdaytotal we got a new day and we have to calculate the summary for yesterday
				#
				#yesterday's mysql date string
				my $yesterday=strftime("%Y-%m-%d", 0, 0, 0, strftime("%d",localtime())-1, strftime("%m",localtime())-1, strftime("%Y",localtime())-1900);
				#do we already have a data record for yesterday?
				$comm = $dbh->prepare("SELECT datum FROM `".$protdata[5]."_total` WHERE datum='".$yesterday."' AND device=".$currentdevice);
				$comm->execute;
				if ($comm->rows==0) { #if there doesn't exist a record for yesterday we didn't make the summary
					#today's timestamp (midnight 00:00:00)
					my $today=mktime(0, 0, 0, strftime("%d",localtime()), strftime("%m",localtime())-1, strftime("%Y",localtime())-1900);
					#get first data record from yesterday
					$comm = $dbh->prepare("SELECT `E-Total` FROM `".$protdata[5]."_".$currentdevice."` WHERE timestamp>=UNIX_TIMESTAMP('".$yesterday."') AND timestamp<'".$today."' ORDER BY timestamp ASC LIMIT 1");
					$comm->execute;
					my $firstrecord=$comm->fetchrow_array();
					#get last data record from yesterday
					$comm = $dbh->prepare("SELECT `E-Total` FROM `".$protdata[5]."_".$currentdevice."` WHERE timestamp>=UNIX_TIMESTAMP('".$yesterday."') AND timestamp<'".$today."' ORDER BY timestamp DESC LIMIT 1");
					$comm->execute;
					my $lastrecord=$comm->fetchrow_array();
					#write the result to database
					$comm = $dbh->prepare("INSERT INTO `".$protdata[5]."_total` SET `E-Total`=".($lastrecord-$firstrecord).", datum='".$yesterday."', device=".$currentdevice);
					$comm->execute;
				}
			}
		}
		#tomorrow's timestamp (midnight 00:00:00)
		#let's remember this value for the main scan loop where we check it again every day about midnight
		$lastdaytotal=mktime(0, 0, 0, strftime("%d",localtime())+1, strftime("%m",localtime())-1, strftime("%Y",localtime())-1900);
	}
}

sub opendebuglog {
	sysopen( LOG, $logfile, O_CREAT | O_APPEND | O_WRONLY, 0640 ) || die "can't write log file ".$logfile;
	LOG->autoflush(1);
	debugprint(0, " -- log file starts here --");
}

sub closedebuglog {
	debugprint(0, " -- log file ends here --");
	close(LOG) || die "can't close log file ".$logfile;
}

sub debugprint {
	my $debuglevel=shift;
	my $msg=shift;
	if (($debug) && ($debuglevel<=$debug)) {
		print LOG strftime("%b %e %H:%M:%S",localtime())." ".$msg."\n";
	}
}

sub createprotocolfile {
	my $currentdevice=shift;
	open(OUT,">".$protocolfileprefix."_".$currentdevice.".".$protocolfilesuffix) || print ("could not create protocol file ".$protocolfileprefix."_".$currentdevice.".".$protocolfilesuffix."\n");
	print OUT join(":",@outputtitlearray)."\n";
	close(OUT);
}

sub rotateprotocolfile {
	my $currentdevice=shift;
	my $filedatestring=shift;
	if (rename($protocolfileprefix."_".$currentdevice.".".$protocolfilesuffix, $protocolfileprefix."_".$currentdevice.".".$protocolfilesuffix.".".$filedatestring)) {
		#write a new file header only after a successful rotation to prevent appearance of a new header line within the protocol file
		createprotocolfile($currentdevice);
	} else {
		debugprint(3, "could not rotate protocol file ".$protocolfileprefix."_".$currentdevice.".".$protocolfilesuffix)
	}
}
##########################################################################
#
# signal_handler
#
# called before exit'ing, when user sent a HUP or INT signal...
#
##########################################################################
sub signal_handler {
	my $sig = shift;

	debugprint(0, "yasdid: signal handler: got signal ".$sig.", goodbye...");

	debugprint(3, "deactivating yasdi drivers...");
	for($i=0;$i<$drivercount;$i++) {
		my $name=yasdi::pl_yasdiGetDriverName(${yasdi::DriverIDArray}[$i]);
		my $result=yasdi::yasdiSetDriverOffline(${yasdi::DriverIDArray}[0]);
		if (!$result) {
			debugprint(2, "Driver ".$name." could not be deactivated.");
		} else {
			debugprint(3, "Driver ".$name." has been deactivated.");
		}
	}

	debugprint(3, "shutting down yasdi...");
	yasdi::yasdiMasterShutdown();

	#disconnect database
	if ($comm) {
		$comm->finish;
	}
	if ($dbh) {
		$dbh->disconnect;
	}

	# remove PID file (-> global lock)
	debugprint(3, "remove global lock ".$pidfile) if $opt_d;
	unlink $pidfile;

	#close logfile
	closedebuglog();

	exit 7;
}

##########################################################################
#
# signal_handler_HUP
#
# called when user sends a HUP signal --> set flag to do graceful exit
#
##########################################################################
sub signal_handler_HUP {
	my $sig = shift;
	debugprint(3, "yasdid: signal handler: got signal ".$sig.", will exit as soon as possible...");
	$graceful_exit_requested = 1;
}

##########################################################################
#
# signal_handler_USR1
#
# called when user sends a USR1 signal --> set flag to roll log file
#
##########################################################################
sub signal_handler_USR1 {
	my $sig = shift;
	debugprint(3, "yasdid: signal handler: got signal ".$sig.", roll log file...");
	$roll_log_file_requested = 1;
}
